import 'rxjs/add/operator/map';
import { Log, Level } from 'ng2-logger/browser';
import { Cookie } from './cookie';
import { Mapping } from './mapping';
import { Models as HelpersModels } from 'typescript-class-helpers/browser';
import { JSON10 } from 'json10/browser';
import { RequestCache } from './request-cache';
import * as _ from 'lodash';
const log = Log.create('rest namespace', Level.__NOTHING);
export var Models;
(function (Models) {
    Models.MethodConfig = HelpersModels.MethodConfig;
    Models.ClassConfig = HelpersModels.ClassConfig;
    Models.ParamConfig = HelpersModels.ParamConfig;
    ;
    ;
    [];
    class BaseBody {
        toJSON(data, isJSONArray = false) {
            let r = isJSONArray ? [] : {};
            if (typeof data === 'string') {
                try {
                    r = JSON.parse(data);
                }
                catch (e) { }
            }
            else if (typeof data === 'object') {
                return data;
            }
            return r;
        }
    }
    Models.BaseBody = BaseBody;
    class HttpBody extends BaseBody {
        constructor(body, isArray = false, entity, circular) {
            super();
            this.body = body;
            this.isArray = isArray;
            this.entity = entity;
            this.circular = circular;
        }
        get booleanValue() {
            return ['ok', 'true'].includes(this.body.trim());
        }
        get json() {
            if (this.entity && typeof this.entity === 'object') {
                const json = this.toJSON(this.body, this.isArray);
                return Mapping.encode(json, this.entity, this.circular);
            }
            let res = this.toJSON(this.body, this.isArray);
            if (this.circular && Array.isArray(this.circular)) {
                res = JSON10.parse(JSON.stringify(res), this.circular);
            }
            return res;
        }
        get text() {
            return this.body.replace(/^\"/, '').replace(/\"$/, '');
        }
    }
    Models.HttpBody = HttpBody;
    class ErrorBody extends BaseBody {
        constructor(data) {
            super();
            this.data = data;
        }
        get json() {
            return this.toJSON(this.data);
        }
        get text() {
            return this.data;
        }
    }
    Models.ErrorBody = ErrorBody;
    class BaseResponse {
        constructor(responseText, headers, statusCode, isArray = false) {
            this.responseText = responseText;
            this.headers = headers;
            this.statusCode = statusCode;
            this.isArray = isArray;
        }
        get cookies() {
            return BaseResponse.cookies;
        }
    }
    BaseResponse.cookies = Cookie.Instance;
    Models.BaseResponse = BaseResponse;
    class HttpResponse extends BaseResponse {
        constructor(sourceRequest, responseText, headers, statusCode, entity, circular, isArray = false) {
            super(responseText, headers, statusCode, isArray);
            this.sourceRequest = sourceRequest;
            this.responseText = responseText;
            this.headers = headers;
            this.statusCode = statusCode;
            this.entity = entity;
            this.circular = circular;
            this.isArray = isArray;
            this.init();
        }
        init() {
            if (typeof this.entity === 'string') {
                // const headerWithMapping = headers.get(entity);
                let entityJSON = this.headers.getAll(this.entity);
                if (!!entityJSON) {
                    this.entity = JSON.parse(entityJSON.join());
                }
            }
            if (typeof this.circular === 'string') {
                // const headerWithMapping = headers.get(circular);
                let circuralJSON = this.headers.getAll(this.circular);
                if (!!circuralJSON) {
                    this.circular = JSON.parse(circuralJSON.join());
                }
            }
            this.body = new HttpBody(this.responseText, this.isArray, this.entity, this.circular);
        }
        get cache() {
            if (_.isUndefined(this.rq)) {
                this.rq = new RequestCache(this);
            }
            return new RequestCache(this);
        }
    }
    Models.HttpResponse = HttpResponse;
    class HttpResponseError extends BaseResponse {
        // public tryRecconect() {
        // }
        constructor(message, responseText, headers, statusCode) {
            super(responseText, headers, statusCode);
            this.message = message;
            this.body = new ErrorBody(responseText);
        }
    }
    Models.HttpResponseError = HttpResponseError;
})(Models || (Models = {}));
//# sourceMappingURL=models.js.map