import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/takeUntil';
import 'rxjs/add/operator/take';
import 'rxjs/add/operator/toPromise';
import { Resource } from './resource.service';
/**
 *
 * @export
 * @abstract
 * @class SimpleResource
 * @extends {Resource<T, A, TA>}
 * @template E  Endpoint type
 * @template A Single modle type
 * @template TA Array Model Type
 * @template RP rest url parameters type
 * @template QP query parameter type
 */
class ExtendedResource {
    // add(endpoint: E, model: string, group?: string, name?: string, description?: string) { }
    constructor(endpoint, path_model) {
        this.endpoint = endpoint;
        this.path_model = path_model;
        /**
         * Get model by rest params
        */
        this.model = (restParams) => {
            return {
                get: (queryPrams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .get([queryPrams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                patch: (item, queryParams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .put(item, [queryParams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                head: (queryPrams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .head([queryPrams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                query: (queryPrams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams).
                            array
                            .get([queryPrams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                post: (item, queryParams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .post(item, [queryParams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                put: (item, queryParams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .put(item, [queryParams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                },
                delete: (queryPrams) => {
                    return Observable.create((observer) => {
                        ExtendedResource.handlers.push(this.rest.model(restParams)
                            .delete([queryPrams], ExtendedResource.doNotSerializeQueryParams)
                            .observable
                            .subscribe(data => observer.next(data.body.json), err => observer.error(err), () => observer.complete()));
                    });
                }
            };
        };
        this.rest = Resource.create(endpoint, path_model);
    }
}
ExtendedResource.doNotSerializeQueryParams = false;
ExtendedResource.handlers = [];
/**
 *
 * @export
 * @class SimpleResource
 * @template A single model type
 * @template TA array model type
 * @template RP rest parameters type
 * @template QP query parameters type
 */
export class SimpleResource {
    constructor(endpoint, model) {
        let rest = new ExtendedResource(endpoint, model);
        this.model = rest.model;
    }
    static set doNotSerializeQueryParams(value) {
        if (!SimpleResource._isSetQueryParamsSerialization) {
            SimpleResource._isSetQueryParamsSerialization = true;
            ExtendedResource.doNotSerializeQueryParams = value;
            return;
        }
        console.warn(`Query params serialization already set as
        ${ExtendedResource.doNotSerializeQueryParams},`);
    }
    static __destroy() {
        ExtendedResource.handlers.forEach(h => h.unsubscribe());
    }
}
SimpleResource._isSetQueryParamsSerialization = false;
//# sourceMappingURL=simple-resource.js.map