'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var RestHeaders = exports.RestHeaders = function () {
    function RestHeaders(headers) {
        var _this = this;

        _classCallCheck(this, RestHeaders);

        /** @internal header names are lower case */
        this._headers = new Map();
        /** @internal map lower case names to actual names */
        this._normalizedNames = new Map();
        if (headers instanceof RestHeaders) {
            headers.forEach(function (values, name) {
                values.forEach(function (value) {
                    return _this.set(name, value);
                });
            });
        } else {
            Object.keys(headers).forEach(function (name) {
                var values = Array.isArray(headers[name]) ? headers[name] : [headers[name]];
                _this.delete(name);
                values.forEach(function (value) {
                    return _this.set(name, value);
                });
            });
        }
    }

    _createClass(RestHeaders, [{
        key: 'append',

        /**
         * Appends a header to existing list of header values for a given header name.
         */
        value: function append(name, value) {
            var values = this.getAll(name);
            if (values === null) {
                this.set(name, value);
            } else {
                values.push(value);
            }
        }
        /**
         * Deletes all header values for the given name.
         */

    }, {
        key: 'delete',
        value: function _delete(name) {
            var lcName = name.toLowerCase();
            this._normalizedNames.delete(lcName);
            this._headers.delete(lcName);
        }
    }, {
        key: 'forEach',
        value: function forEach(fn) {
            var _this2 = this;

            this._headers.forEach(function (values, lcName) {
                return fn(values, _this2._normalizedNames.get(lcName), _this2._headers);
            });
        }
        /**
         * Returns first header that matches given name.
         */

    }, {
        key: 'get',
        value: function get(name) {
            var values = this.getAll(name);
            if (values === null) {
                return null;
            }
            return values.length > 0 ? values[0] : null;
        }
        /**
         * Checks for existence of header by given name.
         */

    }, {
        key: 'has',
        value: function has(name) {
            return this._headers.has(name.toLowerCase());
        }
        /**
         * Returns the names of the headers
         */

    }, {
        key: 'keys',
        value: function keys() {
            return Array.from(this._normalizedNames.values());
        }
        /**
         * Sets or overrides header value for given name.
         */

    }, {
        key: 'set',
        value: function set(name, value) {
            if (Array.isArray(value)) {
                if (value.length) {
                    this._headers.set(name.toLowerCase(), [value.join(',')]);
                }
            } else {
                this._headers.set(name.toLowerCase(), [value]);
            }
            this.mayBeSetNormalizedName(name);
        }
        /**
         * Returns values of all headers.
         */

    }, {
        key: 'values',
        value: function values() {
            return Array.from(this._headers.values());
        }
        /**
         * Returns string of all headers.
         */
        // TODO(vicb): returns {[name: string]: string[]}

    }, {
        key: 'toJSON',
        value: function toJSON() {
            var _this3 = this;

            var serialized = {};
            if (!this._headers) {
                debugger;
            }
            this._headers.forEach(function (values, name) {
                var split = [];
                values.forEach(function (v) {
                    return split.push.apply(split, _toConsumableArray(v.split(',')));
                });
                serialized[_this3._normalizedNames.get(name)] = split;
            });
            return serialized;
        }
        /**
         * Returns list of header values for a given name.
         */

    }, {
        key: 'getAll',
        value: function getAll(name) {
            return this.has(name) ? this._headers.get(name.toLowerCase()) : null;
        }
    }, {
        key: 'mayBeSetNormalizedName',
        value: function mayBeSetNormalizedName(name) {
            var lcName = name.toLowerCase();
            if (!this._normalizedNames.has(lcName)) {
                this._normalizedNames.set(lcName, name);
            }
        }
    }], [{
        key: 'from',
        value: function from(headers) {
            if (!headers) {
                return void 0;
            }
            return new RestHeaders(headers);
        }
        /**
         * Returns a new RestHeaders instance from the given DOMString of Response RestHeaders
         */

    }, {
        key: 'fromResponseHeaderString',
        value: function fromResponseHeaderString(headersString) {
            var headers = new RestHeaders();
            headersString.split('\n').forEach(function (line) {
                var index = line.indexOf(':');
                if (index > 0) {
                    var name = line.slice(0, index);
                    var value = line.slice(index + 1).trim();
                    headers.set(name, value);
                }
            });
            return headers;
        }
    }]);

    return RestHeaders;
}();
//# sourceMappingURL=rest-headers.js.map