import 'rxjs/add/operator/map';
import { Log, Level } from 'ng2-logger/browser';
const log = Log.create('rest.class', Level.__NOTHING);
import { getRestParams, getParamsUrl } from './params';
import { RestHeaders } from './rest-headers';
//#endregion
export const DEFAULT_HEADERS = {
    'Content-Type': 'application/json',
    'Accept': 'application/json'
};
export class Rest {
    constructor(endpoint, request, meta) {
        this.request = request;
        this.meta = meta;
        //#endregion
        //#region  constructor
        this._headers = RestHeaders.from(DEFAULT_HEADERS);
        //#endregion
        this.array = {
            get: (params = void 0, doNotSerializeParams) => {
                return this.req('get', void 0, params, doNotSerializeParams, true);
            },
            head: (params = void 0, doNotSerializeParams) => {
                return this.req('head', void 0, params, doNotSerializeParams, true);
            },
            post: (item, params, doNotSerializeParams) => {
                return this.req('post', item, params, doNotSerializeParams, true);
            },
            put: (item, params, doNotSerializeParams) => {
                return this.req('put', item, params, doNotSerializeParams, true);
            },
            patch: (item, params, doNotSerializeParams) => {
                return this.req('patch', item, params, doNotSerializeParams, true);
            },
            delete: (params, doNotSerializeParams) => {
                return this.req('delete', void 0, params, doNotSerializeParams, true);
            },
            jsonp: (params, doNotSerializeParams) => {
                return this.req('jsonp', void 0, params, doNotSerializeParams, true);
            }
        };
        this.__meta_endpoint = endpoint;
    }
    mock(mock) {
        if ((typeof mock === 'function') || (typeof mock === 'object')) {
            this.mockHttp = mock;
        }
        else {
            throw `[ng2-rest]
      .model(...)
      .mock( < BAD MOCK DATA > )
      ...
      `;
        }
        return this;
    }
    get endpoint() {
        let e = this.__meta_endpoint;
        if (this.restQueryParams !== void 0 && this._endpointRest !== void 0
            && typeof this._endpointRest === 'string' && this._endpointRest.trim() !== '')
            e = this._endpointRest;
        return e;
    }
    set __rest_endpoint(endpoint) {
        this._endpointRest = endpoint;
        if (endpoint === void 0) {
            this.restQueryParams = void 0;
        }
        else {
            this.restQueryParams = getRestParams(endpoint, this.__meta_endpoint);
        }
    }
    creatUrl(params, doNotSerializeParams = false) {
        return `${this.endpoint}${getParamsUrl(params, doNotSerializeParams)}`;
    }
    get headers() {
        return this._headers;
    }
    //#endregion
    //#region  req
    req(method, item, params, doNotSerializeParams = false, isArray = false) {
        const modelUrl = this.creatUrl(params, doNotSerializeParams);
        const body = item ? JSON.stringify(item) : void 0;
        const result = this.request[method.toLowerCase()](modelUrl, body, this.headers, this.meta, isArray, this.mockHttp);
        this._headers = RestHeaders.from(DEFAULT_HEADERS);
        this.mockHttp = void 0;
        return result;
    }
    //#endregion
    //#region http methods
    //#region replay
    replay(method) {
        this.request.replay(method, this.meta);
    }
    get(params, doNotSerializeParams = false) {
        return this.req('get', void 0, params, doNotSerializeParams);
    }
    head(params, doNotSerializeParams = false) {
        return this.req('head', void 0, params, doNotSerializeParams);
    }
    post(item, params, doNotSerializeParams = false) {
        return this.req('post', item, params, doNotSerializeParams);
    }
    put(item, params, doNotSerializeParams = false) {
        return this.req('put', item, params, doNotSerializeParams);
    }
    patch(item, params, doNotSerializeParams = false) {
        return this.req('patch', item, params, doNotSerializeParams);
    }
    delete(params, doNotSerializeParams = false) {
        return this.req('delete', void 0, params, doNotSerializeParams);
    }
    jsonp(params, doNotSerializeParams = false) {
        return this.req('jsonp', void 0, params, doNotSerializeParams);
    }
}
//# sourceMappingURL=rest.class.js.map